package cn.fastfun.controller.api;

import cn.fastfun.controller.dto.DeviceAttrDTO;
import cn.fastfun.controller.param.DeviceBathQueryParam;
import cn.fastfun.controller.param.DeviceQueryParam;
import cn.fastfun.service.AppDeviceService;
import cn.fastfun.service.SysExcelFieldService;
import cn.fastfun.service.entity.AppDevice;
import cn.fastfun.service.entity.Product;
import cn.fastfun.util.DateUtils;
import cn.fastfun.util.VerifyUtil;
import cn.fastfun.util.ObjectUtil;
import com.bridge.dto.ApiDTO;
import com.bridge.dto.ApiPageDTO;
import com.bridge.dto.IdParam;
import com.bridge.dto.QueryParamExp;
import com.bridge.service.JpaService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Bridge AutoGen
 */
@Slf4j
@Api(tags = {"设备信息"})
@RestController
@RequestMapping("/api/v1/appdevice")
public class ApiAppDeviceController {
    //业务类
    @Resource(name = "appDeviceService")
    AppDeviceService appDeviceService;

    @Resource
    SysExcelFieldService sysExcelFieldService;

    /**
     * 数据保存
     */
    @ApiOperation(value = "单个电池保存信息")
    @RequestMapping(value = "save", method = RequestMethod.POST)
    public ApiDTO save(@RequestBody @ApiParam(name = "设备信息对象", value = "传入json格式", required = true) AppDevice entity) {
        // 此处定义的sn实际上是指的表中的imei，也就是实体类的属性imei
        String checkSnResult = VerifyUtil.checkSn(entity.getImei());
        List<AppDevice> checkDuplicatedResultOnImei = appDeviceService.findAll(
                QueryParamExp.eq("imei", entity.getImei()));
        if (!checkDuplicatedResultOnImei.isEmpty()) {
            return ApiDTO.error(500, "该imei已经被录入", checkDuplicatedResultOnImei);
        }
        List<AppDevice> checkDuplicatedResultOnSn = appDeviceService.findAll(
                QueryParamExp.eq("sn", entity.getSn()));
        if (!checkDuplicatedResultOnSn.isEmpty()) {
            return ApiDTO.error(500, "该sn已经被录入", checkDuplicatedResultOnSn);
        }
        if (checkSnResult.equals("passed")) {
            AppDevice tmp = appDeviceService.save(entity);
            return ApiDTO.ok("保存成功", tmp);
        } else {
            return ApiDTO.error(500, checkSnResult);
        }
    }


    @ApiOperation(value = "查询信息")
    @RequestMapping(value = "get", method = RequestMethod.POST)
    public ApiDTO get(@RequestBody @ApiParam(name = "批次编号", required = true) IdParam param) {
        List<AppDevice> appDevicesOnOneBatch = appDeviceService.findAll(
                QueryParamExp.eq("batchNum", param.getId()));
        return ApiDTO.ok("查询成功！", appDevicesOnOneBatch);
    }

//    @ApiOperation(value = "删除信息")
//    @RequestMapping(value = "delete", method = RequestMethod.POST)
//    public ApiDTO delete(@RequestBody @ApiParam(name = "设备信息id", required = true) IdParam param) {
//        appDeviceService.delete(Arrays.asList(param.getId().split(",")));
//        return ApiDTO.ok("删除成功！");
//    }

    @ApiOperation(value = "分页搜索")
    @RequestMapping(value = "pageQuery", method = RequestMethod.POST)
    public ApiPageDTO pageQuery(@RequestBody DeviceQueryParam param) {
        return new ApiPageDTO(null, appDeviceService.findByParam(param));
    }

    @ApiOperation(value = "导入模板下载")
    @GetMapping("excelTemplate")
    public void excelTemplate(HttpServletResponse response, @RequestParam String name) {
        HSSFWorkbook workbook = sysExcelFieldService.getTemplate("invoice");
        sysExcelFieldService.outExcelFile(workbook, response, name);
    }

    @ApiOperation(value = "数据导入")
    @RequestMapping(value = "excelImport", method = RequestMethod.POST)
    public ApiDTO excelImport(MultipartFile file) {
        appDeviceService.importDevice(sysExcelFieldService.importExcel("invoice", file));
        return ApiDTO.ok();
    }

    @ApiOperation(value = "数据计数")
    @RequestMapping(value = "count", method = RequestMethod.POST)
    public ApiDTO excelImport() {
        Long deviceCount = appDeviceService.getCount();
        return ApiDTO.ok(deviceCount);
    }

    @ApiOperation(value = "设备批次分页搜索")
    @RequestMapping(value = "batchNum/pageQuery", method = RequestMethod.POST)
    public ApiPageDTO batchNumPageQuery(@RequestBody DeviceBathQueryParam param) {
        StringBuffer sql = new StringBuffer("select t1.batch_num,t1.sn,t1.deliver_time,count(0) as total,t1.add_time,t1.deliver_addr from app_device t1");
        sql.append(" group by t1.batch_num");
        if (!StringUtils.isEmpty(param.getOrderBy())) {
            sql.append(param.getOrderBy());
        }
        log.info("SQL: {}", sql.toString());


        ApiPageDTO page = appDeviceService.getListBySQL(sql.toString(), new HashMap<>(), param);
        // 组装数据
        List<Map<String, Object>> list = (List<Map<String, Object>>) page.getData();
        for (Map<String, Object> p : list) {
            DeviceAttrDTO attr = appDeviceService.sn2Attr(ObjectUtil.obj2String(p.get("sn")));
            p.put("type_title", attr.getTypeTitle());
            p.put("pack_title", attr.getPackTitle());
            p.put("deliver_time", DateUtils.toString(ObjectUtil.obj2Date(p.get("deliver_time")), DateUtils.YMD));
            p.put("add_time", DateUtils.toString(ObjectUtil.obj2Date(p.get("add_time")), DateUtils.YMDHMS));
            p.put("in_storage", 0);
        }
        page.setData(list);
        return page;
    }

}